//******************************************************************************//
//																				//
//		Project	:	Quad Copter													//
//		Device	:	PSoC 4														//
//		Version :	0.0															//
//		Date	:	March 2014													//
//		Author(s)	Bob Marlowe													//
//																				//
//******************************************************************************//

//******************************************************************************//
//********************* Libraries **********************************************//
//******************************************************************************//
#include <DeMPX.h>
#include <project.h>

//******************************************************************************//
//********************* Definitions & Macros ***********************************//
//******************************************************************************//
#define TicksPer_ms		120							// Clock runs at 120kHz
#define	SyncPulseLength 290	* TicksPer_ms			// Syncpulse is 300ms
#define	LowPulseLength  1	* TicksPer_ms			// Lowest is 1ms
#define	HighPulseLength	2	* TicksPer_ms			// Longest is 2ms

//******************************************************************************//
//********************* Data Area **********************************************//
//******************************************************************************//
uint16 Channel[NumberOfChannels];					// Decoded channel values from RC
uint8 MPX_SignalLost;								// No Signal detected initially

//******************************************************************************//
//********************* Prototypes **********************************************//
//******************************************************************************//
CY_ISR_PROTO(MPXHandler);


//******************************************************************************//
//********************* Program Area *******************************************//
//******************************************************************************//
void MPX_Start(void)								// Start the De-Multiplexer
{
	MPXInt_StartEx(MPXHandler);
	MPX_Discriminator_Start();
}
//**********************************************************************
void MPX_Stop(void)									// Stop it
{
	
}
//**********************************************************************

CY_ISR(MPXHandler)
{
static int16 ChCount = -1;							// Indicate No Sync-pulse found
int32 CapValue;
	if(MPX_Discriminator_GetInterruptSource() & MPX_Discriminator_INTR_MASK_TC)
	{												// We lost Signal!! Indicate that, then exit
		ChCount = -1;
		MPX_SignalLost = 1;
		return;
	}
	if(ChCount >= SyncPulseLength)					// Do we read a synchronize pulse
	{
		ChCount = 0;								// Startz with first channel
		MPX_SignalLost = 0;							// Signal we saw a sync-pulse
		return;
	}
	
	MPX_Discriminator_Stop();						// Stop MPX to reset counter to zero
	MPX_Discriminator_WriteCounter(0);
	CapValue = MPX_Discriminator_ReadCapture();		// Get signal high-time..
	MPX_Discriminator_Start();						// ..and restart counter
	
	if(ChCount >= NumberOfChannels)					// More channels than defined
	{
		return;
	}
	if(ChCount < 0) 								// If not yet in sync
	{
		return;										// We are not in sync yet
	}
	
//	Adjust the read value from 2..20ms to 0..999
	CapValue -= LowPulseLength;
	CapValue *= 10;
	if(CapValue < 0) CapValue = 0;
	if(CapValue >= 999) CapValue = 999;
	Channel[ChCount++] = CapValue;
}
//**********************************************************************

