//******************************************************************************//
//																				//
//		Project	:	Quad Copter													//
//		Device	:	PSoC 4														//
//		Version :	0.0															//
//		Date	:	March 2014													//
//		Author(s)	Bob Marlowe													//
//																				//
//******************************************************************************//

//******************************************************************************//
//********************* Libraries **********************************************//
//******************************************************************************//
#include <GlobalDefs.h>
#include <project.h>
#include <CyLib.h>
//******************************************************************************//
//********************* Definitions & Macros ***********************************//
//******************************************************************************//
#define I2C_READ		1						// Some I2C constants
#define I2C_WRITE		0
#define I2C_ACKslave	1
#define I2C_NAKslave	0

#define Accell_I2C	 		0x19				//	Slave addresses Accellerometer
#define Magnet_I2C	    	0x1e				//	Compass
#define Gyro_I2C		 	0x6b				//	Gyro

//************* L3G4200 Register Definitions **********************************
#define TempRegisterGyro 	0x26
#define	WhoAmIGyro			0x0f
#define	CtlReg1Gyro			0x20
#define	CtlReg2Gyro			0x21
#define	CtlReg3Gyro			0x22
#define	CtlReg4Gyro			0x23
#define	CtlReg5Gyro			0x24
#define AchseX_Gyro 	 	0x28
#define AchseY_Gyro   		0x2A
#define AchseZ_Gyro   		0x2C
#define Status_Reg_Gyro     0x27
#define	FIFO_CtlReg_Gyro	0x2e
#define FIFO_StatReg_Gyro	0x2f
#define DataReadyBit_Gyro	3

#define GyroFullscale		500.0				// Fullscale in °/s
//	Für 760 Samples addiert, 1 Sekunde lang, bei 500 °/s ergibt das 32768 * 760
//  bei 1°/s ist es also 32768 * 760 / 500 
#define DFactor 			(GyroFullscale  / (32768.0 * GyroFrequ)) // Der Kehrwert

#define DataReadyBit		3
#define DataReadyBit_Mag 	0
#define DataOverwritten 	7

//***** LSM303 Register Definitions ********
#define TempRegister 	0x26
#define	WhoAmI			0x0f
#define	CtlReg1			0x20
#define	CtlReg2			0x21
#define	CtlReg3			0x22
#define	CtlReg4			0x23
#define	CtlReg5			0x24
#define	HP				0x26
#define StatusReg       0x27
#define AchseX  	 	0x28
#define AchseY   		0x2A
#define AchseZ   		0x2C

//****** LSM303 Kompass********

#define CRA_REG_M 		0x00
#define CRB_REG_M       0x01
#define MR_REG_M        0x02

#define mag_x			0x03
#define mag_z			0x05
#define mag_y			0x07

#define SR_REG_M			0x09


//******************************************************************************//
//********************* Data Area **********************************************//
//******************************************************************************//
float Gyro_X, Gyro_Y, Gyro_Z;
float Magneto_X, Magneto_Y, Magneto_Z;
float Accellmeter_X, Accellmeter_Y, Accellmeter_Z;

int32 ResponseX_Gyro;
int32 ResponseY_Gyro;
int32 ResponseZ_Gyro;

int32 ResponseX_Mag;
int32 ResponseY_Mag;
int32 ResponseZ_Mag;

int32 ResponseX_Acc;
int32 ResponseY_Acc;
int32 ResponseZ_Acc;

int32 G_OffsetX;								// Measured Gyro Offsets
int32 G_OffsetZ;
int32 G_OffsetY;

int32 A_OffsetX;								// Measured Accellerometer Offsets
int32 A_OffsetZ;
int32 A_OffsetY;
uint16 ACCSampleCount;							// Counter for Average

//******************************************************************************//
//********************* Prototypes **********************************************//
//******************************************************************************//
void InitializeGyroReadings(void);
void AddAccTuple(void);
void SetAccOffsets(uint16);
void SetGyroOffsets(uint16);
//******************************************************************************//
//********************* Program Area *******************************************//
//******************************************************************************//

uint8 I2C_ReadByte(uint8 AckNack)
{
	return I2C_I2CMasterReadByte(AckNack);
}
//******************************************************************************//

void I2C_PreAmble(uint8 Device, uint8 Register)
{
	I2C_I2CMasterSendStart(Device, Register);
	I2C_I2CMasterWriteByte(Register);
	I2C_I2CMasterSendRestart(Device,I2C_READ);
}
//******************************************************************************//

void WriteGeberByte(uint8 I2C_Device, uint8 Register, uint8 Value)
{
	I2C_I2CMasterSendStart(I2C_Device, I2C_WRITE);
	I2C_I2CMasterWriteByte(Register);
	I2C_I2CMasterWriteByte(Value);
	I2C_I2CMasterSendStop();
}
//******************************************************************************//

uint8 ReadGeberByte(uint8 I2C_Device, uint8 Register)
{
uint8 Status;
	I2C_PreAmble(I2C_Device,Register);
	Status = I2C_I2CMasterReadByte(I2C_NAKslave);
	I2C_I2CMasterSendStop();
	return Status;
}
//******************************************************************************//

void Wait(void)
{
	if(!IsBitSet(ReadGeberByte(Accell_I2C,StatusReg),DataReadyBit)) return;
	
	AddAccTuple();
	
}
//******************************************************************************//

void Accell_Start()
{
	WriteGeberByte(Accell_I2C,0x20,				0b01110111);	// Reg 0: Power up Beschleunigungs-Geber, 400Hz 
	WriteGeberByte(Accell_I2C,0x23,				0b10000000);	// Reg 4: Enable Block Write
	
	SetAccOffsets(100);
}
//******************************************************************************//

void Magnet_Start()
{
	WriteGeberByte(Magnet_I2C,0x00,				0b10011000);	// Reg 0: Enable Temperature; 75 Hz, Normal
	WriteGeberByte(Magnet_I2C,0x01,				0b00100000);	// Reg 1: Higest Gain
	WriteGeberByte(Magnet_I2C,0x02,				0b00000000);	// Reg 2: Mode: Continuous Conversion
}
//******************************************************************************//

void Gyro_Start(void)
{
	WriteGeberByte(Gyro_I2C,CtlReg1Gyro,		0b11011111);	// Standard 760 Smpls/s, 30Hz LPF
	WriteGeberByte(Gyro_I2C,FIFO_CtlReg_Gyro,	0b01001111);	// Streaming mode,  Watermark set to 15
	WriteGeberByte(Gyro_I2C,CtlReg4Gyro,		0b10010000);	// Block Mode Scale 500°/s
	WriteGeberByte(Gyro_I2C,CtlReg5Gyro,		0b01000000);	// Enable FIFO
	
	SetGyroOffsets(250);
}
//******************************************************************************//

void Sensors_Start(void)
{
	I2C_Start();
	Accell_Start();
	Magnet_Start();
	Gyro_Start();
}
//******************************************************************************//

uint8 WaitForGyroData(void)
{
uint8 Count;
	while((Count = ReadGeberByte(Gyro_I2C,FIFO_StatReg_Gyro) & 0x1f) == 0)
	{
		Wait();  
	}
	return Count;
}

//******************************************************************************//
void Get3Tuples(uint8 Number)
{
uint8 datalow;
int16 datahigh;
int16 Intermed;

	I2C_PreAmble(Gyro_I2C, AchseX_Gyro | 0x80);			// High bit on signals reading multiple values from FIFO

	while(Number > 0)
	{
		
		Number--;

		datalow		= I2C_ReadByte(I2C_ACKslave);
		datahigh	= I2C_ReadByte(I2C_ACKslave);	
		Intermed	= (datahigh << 8) | datalow;
		ResponseX_Gyro += (int32)Intermed - G_OffsetX;	

		datalow		= I2C_ReadByte(I2C_ACKslave);
		datahigh	= I2C_ReadByte(I2C_ACKslave);	
		Intermed	= (datahigh << 8) | datalow;
		ResponseY_Gyro += (int32)Intermed - G_OffsetY;	

		datalow		= I2C_ReadByte(I2C_ACKslave);
		datahigh	= I2C_ReadByte(Number > 0?I2C_ACKslave:I2C_NAKslave);	
		Intermed	= (datahigh << 8) | datalow;
		ResponseZ_Gyro += (int32)Intermed - G_OffsetZ;	
	}
	I2C_I2CMasterSendStop();
}
//******************************************************************************//

void InitializeGyroReadings(void)
{
	G_OffsetX = 0;
	G_OffsetY = 0;
	G_OffsetZ = 0;
	ResponseX_Gyro = 0;
	ResponseY_Gyro = 0;
	ResponseZ_Gyro = 0;
}
//******************************************************************************//

void SetGyroOffsets(uint16 SampleCount)
{
uint16 SumSamples = 0;
uint16 Actual;
	InitializeGyroReadings();
	while(SumSamples < SampleCount)
	{
		SumSamples += Actual = WaitForGyroData();
	}
	SampleCount = SumSamples;
	G_OffsetX = ResponseX_Gyro / SampleCount;
	G_OffsetY = ResponseY_Gyro / SampleCount;
	G_OffsetZ = ResponseZ_Gyro / SampleCount;
	ResponseX_Gyro = 0;
	ResponseY_Gyro = 0;
	ResponseZ_Gyro = 0;
}
//******************************************************************************//

void Gyro_GetValues(void)
{
uint8 nn;	
	nn = WaitForGyroData();						// Wait for at least one data tuple Ready
		
	Get3Tuples(nn);								// Read FIFO and integrate

	Gyro_X = (float)ResponseX_Gyro /  nn;		// Average integrated values
	Gyro_Y = (float)ResponseY_Gyro /  nn;
	Gyro_Z = (float)ResponseZ_Gyro /  nn;
	
	ResponseX_Gyro = 0;							// Reset the integral of Gyro-values
	ResponseY_Gyro = 0;
	ResponseZ_Gyro = 0;
	
}
//******************************************************************************//

unsigned char MagDataReady(void)
{
	return IsBitSet(ReadGeberByte(Magnet_I2C,SR_REG_M),DataReadyBit_Mag);
}

int16 GetAxisMag(uint8 Axis)
{
uint8 datalow;
int16 datahigh;
int16 Intermed;

	I2C_PreAmble(Magnet_I2C, Axis);			// Initialize read of Axix

	datalow		= I2C_ReadByte(I2C_ACKslave);
	datahigh	= I2C_ReadByte(I2C_ACKslave);	
	Intermed	= (datahigh << 8) | datalow;
	I2C_I2CMasterSendStop();
	
	return Intermed;
}
//******************************************************************************//

void Magnet_GetValues(void)				//Magnetfeldsensor
{
	while(!MagDataReady())
	{
		Wait();							// Warten bis Daten bereit
	}
	ResponseX_Mag = GetAxisMag(mag_x);
	ResponseY_Mag = GetAxisMag(mag_y);
	ResponseZ_Mag = GetAxisMag(mag_z);
}
//******************************************************************************//

void AddAccTuple(void)
{
uint8 datalow;
int16 datahigh;
int16 Intermed;

	I2C_PreAmble(Accell_I2C, AchseX);			// Initialize read of Axix

	datalow		= I2C_ReadByte(I2C_ACKslave);
	datahigh	= I2C_ReadByte(I2C_ACKslave);	
	Intermed	= (datahigh << 8) | datalow;
	ResponseX_Acc += (int32)Intermed - A_OffsetX;
	
	datalow		= I2C_ReadByte(I2C_ACKslave);
	datahigh	= I2C_ReadByte(I2C_ACKslave);	
	Intermed	= (datahigh << 8) | datalow;
	ResponseY_Acc += (int32)Intermed - A_OffsetY;
	
	datalow		= I2C_ReadByte(I2C_ACKslave);
	datahigh	= I2C_ReadByte(I2C_ACKslave);	
	Intermed	= (datahigh << 8) | datalow;
	ResponseZ_Acc += (int32)Intermed - A_OffsetZ;
	
	I2C_I2CMasterSendStop();
	ACCSampleCount++;
}
//******************************************************************************//

void InitializeAccReadings(void)
{
	A_OffsetX = 0;
	A_OffsetY = 0;
	A_OffsetZ = 0;
	ResponseX_Acc = 0;
	ResponseY_Acc = 0;
	ResponseZ_Acc = 0;
	ACCSampleCount = 0;
}
//******************************************************************************//

void SetAccOffsets(uint16 SamplesWanted)
{
	InitializeAccReadings();
	while(SamplesWanted < ACCSampleCount)
	{
		Wait();;
	}
	A_OffsetX = ResponseX_Acc / ACCSampleCount;
	A_OffsetY = ResponseY_Acc / ACCSampleCount;
	A_OffsetZ = ResponseZ_Acc / ACCSampleCount;
	ResponseX_Acc = 0;
	ResponseY_Acc = 0;
	ResponseZ_Acc = 0;
	ACCSampleCount = 0;
}
//******************************************************************************//

void Accell_GetValues(void)
{
	while(!ACCSampleCount) Wait();			// Wait for at least 1 sample ready
	Accellmeter_X = (float)ResponseX_Acc / ACCSampleCount;
	Accellmeter_Y = (float)ResponseY_Acc / ACCSampleCount;
	Accellmeter_Z = (float)ResponseZ_Acc / ACCSampleCount;
	
	ACCSampleCount = 0;
}
//******************************************************************************//

void Sensors_GetAll(void)
{
	Accell_GetValues();	
	Magnet_GetValues();
	Gyro_GetValues();
}
//******************************************************************************//
