//******************************************************************************//
//																				//
//		Project	:	Quad Copter													//
//		Device	:	PSoC 4														//
//		Version :	0.0															//
//		Date	:	March 2014													//
//		Author(s)	Bob Marlowe													//
//																				//
//******************************************************************************//

//******************************************************************************//
//																				//
//	This project is the attempt to control a quad copter with a PSoC4 chip		//
//																				//
//	The underlying hardware can be as small as the CY8CKIT-049-42xx-board 		//
//	which with a weight of less than 10g puts a rather small load to a flying 	//
//	object. The motor-controll uses all four UDBs, the communication to the 	//
//	sensors is made up with I2C.												//
//	The requred MEMS sensors are: 3-axis magnet for the orientation of yaw		//
//	a 3-axis gyro to detect movements and a 2 or 3-axis accellerometer to 		//
//	detect the pitch and roll-angles and a yaw acceleration.					//
//																				//
//	I would like to point out that the project can be developed and debugged 	//
//	with the Pioneer board and later programmed into a KIT-049. Debugging on a	//
//	KIT-049 is possible only with the help of a MiniProg3.						//
//																				//
//******************************************************************************//
//																				//
//	The current design controls 4 motors via the standard 1 to 2ms rc interface	//
//	and uses a so-called multiplex sum-signal as input which is a standard		//
//	for copters, too.															//
//	The targeted update frequency is 50Hz, which could get tough for the		//
//	floating-point math used for the AHRS algorithm AND for the time needed		//
//	to transfer the data from the sensors which should be averaged to reduce	//
//	noise that will get more caused by vibrations from rotating motors.			//
//																				//
//	This configuration already uses *ALL* UDBs, Datapaths, control- and 		//
//	status-registers, so there is no chance controlling a hex-copter			//
//																				//
//******************************************************************************//



//******************************************************************************//
//********************* Libraries **********************************************//
//******************************************************************************//
#include <project.h>
#include <GlobalDefs.h>
#include <DeMPX.h>
#include <Sensors.h>
#include <Motors.h>
#include <Mayhony.h>

//******************************************************************************//
//********************* Prototypes **********************************************//
//******************************************************************************//
CY_ISR_PROTO(Handler50Hz);				//	Interrupt for the main-loop

//******************************************************************************//
//********************* Data Area **********************************************//
//******************************************************************************//
uint8 WaitFor50Hz	= TRUE;				//	Indicator for Main clock

//******************************************************************************//
//********************* Program Area *******************************************//
//******************************************************************************//
void InitializeHardware(void)
{
	CyGlobalIntEnable;					//	
	Timer50Hz_Start();					//	Main clock is running at 50Hz
	Int50Hz_StartEx(Handler50Hz);		//	
	MPX_Start();						//	Start the De-Multiplexing of RC-signal
	Sensors_Start();	
}
//******************************************************************************//

void Mixer(void)
{
	// Purpose is to distribute the 3 AHRS values together with
	// the incoming RC for yaw,pitch,roll and up/down
	// to the 4 motor controls of the copter.
}

int main()
{
	InitializeHardware();				//	Startup everything
    while(forever)						//	Main loop 
    {
		IndicatorLED_Write(FALSE);		//	Switch LED off to show idle		
		while(WaitFor50Hz) Wait();		//	Wait until 50Hz timer expires
		IndicatorLED_Write(TRUE);		//	Switch LED on to show processing
		
		Sensors_GetAll();
		MayhonyAHRSupdate(Gyro_X,Gyro_Y,Gyro_Z,Accellmeter_X,Accellmeter_Y,Accellmeter_Z,Magneto_X,Magneto_Y,Magneto_Z);
		
		Mixer();
		UpdateAllMotors();
		WaitFor50Hz = TRUE;
		
	}
}
//******************************************************************************//

CY_ISR(Handler50Hz)
{
	WaitFor50Hz = FALSE;				//	Indicate timer occured
}
//******************************************************************************//
